/*
This file is part of 'hamster-shell-extension'.

'hamster-shell-extension' is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

'hamster-shell-extension' is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with 'hamster-shell-extension'.  If not, see <http://www.gnu.org/licenses/>.

Copyright (c) 2011 Jerome Oufella <jerome@oufella.com>
Copyright (c) 2011-2012 Toms Baugis <toms.baugis@gmail.com>
Icons Artwork Copyright (c) 2012 Reda Lazri <the.red.shortcut@gmail.com>
Copyright (c) 2016 - 2018 Eric Goller / projecthamster <elbenfreund@projecthamster.org>
*/


.hamster-box {
    spacing: 15px;
    min-width: 25em;
}

.hamster-box-label {
    padding-top: 1em;
    text-align: center;
}

.hamster-activities {
    border: 1px solid #333;
}

.hamster-scrollbox {
    max-height: 200px;
    overflow-x: none;
    overflow-y: auto;
}

.cell-label {
    padding: 3px;
    padding-right: 15px;
    color: #aaa;
}

.summary-label {
    color: #aaa;
    text-align: right;
}

.cell-button {
    padding: 3px 5px;
}

.clickable:hover {
    background-color: #4c4c4c;
}

.clickable:hover .cell-label {
    color: #fff;
}

.panel-box {
    spacing: 5px;
}
